@@ -2,7 +2,7 @@
 package MRTG_lib;
 
 ###################################################################
-# MRTG 2.9.10  Support library MRTG_lib.pm
+# MRTG 2.9.12a  Support library MRTG_lib.pm
 ###################################################################
 # Created by Tobias Oetiker <oetiker@ee.ethz.ch>
 #            and Dave Rand <dlr@bungi.com>
@@ -44,7 +44,7 @@ require Exporter;
              demonize_me debug log2rrd storeincache
 	     populateconfcache readconfcache writeconfcache);
 
-$VERSION = 2.090009;
+$VERSION = 2.090011;
 
 
 %mrtgrules =
@@ -110,6 +110,10 @@ $VERSION = 2.090009;
        'nospacechar' =>
        [sub{length($_[0]) == 1}, sub{"$_[0] must be one character long"}],
 
+       'snmpoptions' =>
+       [sub{ eval( '{'.$_[0].'}' ); return not $@},
+        sub{"Must have the format \"OptA => Number, OptB => 'String', ... \""}],
+
        # Per Router CFG
        'target[]' => 
        [sub{1}, sub{"Internal Error"}], #will test this later
@@ -306,6 +310,7 @@ sub readcfg ($$$$;$$) {
     my $file;
     my @filestack;
     local *CFG;
+    if ($cfgfile eq '-'){$cfgfile = '<&STDIN'};
     open (CFG, $cfgfile) || die "ERROR: unable to open config file: $cfgfile\n";
     $hand = *CFG;
     my @handstack;
@@ -521,7 +526,9 @@ sub cfgcheck ($$$$) {
     my ($rou, $confname, $one_option);
     my $error="no";
     my(@known_options) = qw(growright bits noinfo absolute gauge nopercent
-			    integer perhour perminute transparent dorelpercent unknaszero withzeroes);
+			    integer perhour perminute transparent dorelpercent 
+			    unknaszero withzeroes noborder noarrow noi noo
+			    nobanner nolegend);
 
     if (defined $$cfg{workdir}) {
         ensureSL(\$$cfg{workdir});
@@ -532,8 +539,8 @@ sub cfgcheck ($$$$) {
         }
         
     } elsif ( not (defined $$cfg{logdir} or defined $$cfg{htmldir} or defined $$cfg{imagedir})) {
-          warn ("WARNING: \"WorkDir\" not specified\n");
-          $error = "yes";
+          die ("ERROR: \"WorkDir\" not specified in mrtg config file\n");
+	  $error = "yes";
     } else {
         if (! defined $$cfg{logdir}) {
             warn ("WARNING: \"LogDir\" not specified\n");
@@ -620,6 +627,12 @@ sub cfgcheck ($$$$) {
         die "ERROR: could not find RRDs.pm. Use LibAdd: in mrtg.cfg to help mrtg find RRDs.pm\n" 
                 unless defined $found;
     }
+    if (defined $$cfg{snmpoptions}) {
+           $$cfg{snmpoptions} = eval('{'.$$cfg{snmpoptions}.'}');
+    } else {
+           my %hash;
+           $$cfg{snmpoptions} = \%hash;
+    }
 
     # default interval is 5 minutes
     $$cfg{interval} = 5 unless defined $$cfg{interval};
@@ -663,6 +676,8 @@ sub cfgcheck ($$$$) {
             # allow for linebreaks
             $$rcfg{"pagetop"}{$rou} =~ s/\\n/\n/g;
         }
+
+
         if (defined $$rcfg{"pagefoot"}{$rou}) {
             # allow for linebreaks
             $$rcfg{"pagefoot"}{$rou} =~ s/\\n/\n/g;
@@ -1191,39 +1206,41 @@ sub storeincache ($$$$$){
 
 }
 
-sub populateconfcache ($$$) {
+sub populateconfcache ($$$$) {
     my $confcache = shift;
     my $host = shift;
     my $reread = shift;
+    my $snmpoptions = shift || {};
+
     return if defined $$confcache{$host} and not $reread;
     # clear confcache for host;
     delete $$confcache{$host};
-    my @ret = snmpwalk($host, "ifDescr");
+    my @ret = snmpwalk($host, $snmpoptions, "ifDescr");
     foreach my $ret (@ret)
       {
 	  my ($oid, $desc) = split(':', $ret, 2);
 	  storeincache($confcache,$host,"Descr",$desc,$oid);
       };
-    @ret = snmpwalk($host, "ifName");
+    @ret = snmpwalk($host, $snmpoptions, "ifName");
     foreach my $ret (@ret)
       {
 	  my ($oid, $desc) = split(':', $ret, 2);
 	  storeincache($confcache,$host,"Name",$desc,$oid);
       };
-    @ret = snmpwalk($host, "ifType");
+    @ret = snmpwalk($host, $snmpoptions, "ifType");
     foreach my $ret (@ret)
       {
 	  my ($oid, $desc) = split(':', $ret, 2);
 	  storeincache($confcache,$host,"Type",$desc,$oid);
       };
-    @ret = snmpwalk($host, "ipAdEntIfIndex");
+    @ret = snmpwalk($host, $snmpoptions, "ipAdEntIfIndex");
     foreach my $ret (@ret)
       {
 	  my ($addr, $oid) = split(':', $ret, 2);
 	  storeincache($confcache,$host,"Ip",$addr,$oid);
       };
 
-    @ret = snmpwalk($host, "ifPhysAddress");
+    @ret = snmpwalk($host, $snmpoptions, "ifPhysAddress");
     foreach my $ret (@ret)
       {
 	  my ($oid, $bin) = split(':', $ret, 2);
@@ -1612,7 +1629,7 @@ Puts the running program into background, detaching it from the terminal.
 
 =item C<populatecache>
 
-C<populatecache(\%confcache, $host, $reread)>
+C<populatecache(\%confcache, $host, $reread, $snmpoptshash)>
 
 Reads the SNMP variables I<ifDescr>, I<ipAdEntIfIndex>, I<ifPhysAddress>, I<ifName> from
 the I<host> and stores the values in I<%confcache> as follows:
