@@ -15,21 +15,25 @@
 ### Simon Leinen <simon@switch.ch>: SNMP_session.pm/BER.pm
 ### Jeff Allen <jeff.allen@acm.org>: length() of undefined value
 ### Johannes Demel <demel@zid.tuwien.ac.at>: MIB file parse problem
+### Simon Leinen <simon@switch.ch>: more OIDs from Interface MIB
+### Jacques Supcik <supcik@ip-plus.net>: Specify local IP, port
+### Tobias Oetiker <oetiker@ee.ethz.ch>: HASH as first OID to set SNMP options
 ######################################################################
 
 package SNMP_util;
 
-require 5.002;
+require 5.004;
 
 use strict;
 use vars qw(@ISA @EXPORT $VERSION);
 use Exporter;
+use Carp;
 
 use BER "0.82";
 use SNMP_Session "0.83";
 use Socket;
 
-$VERSION = '0.77';
+$VERSION = '0.85';
 
 @ISA = qw(Exporter);
 
@@ -317,6 +321,7 @@ my $agent_start_time = time;
 undef $SNMP_util::Host;
 undef $SNMP_util::Session;
 undef $SNMP_util::Version;
+undef $SNMP_util::LHost;
 $SNMP_util::Debug = 0;
 $SNMP_util::CacheFile = "OID_cache.txt";
 $SNMP_util::CacheLoaded = 0;
@@ -344,23 +349,35 @@ sub version () { $VERSION; }
 # Start an snmp session
 #
 sub snmpopen (@) {
-  my($host) = @_;
-  my($nhost, $port, $community);
+  my($host, $type, $vars) = @_;
+  my($nhost, $port, $community, $lhost, $lport, $nlhost);
   my($timeout, $retries, $backoff, $version);
 
+  $type = 0 if (!defined($type));
   $community = "public";
-  $port = 161;
+  $nlhost = "";
 
   ($community, $host) = ($1, $2) if ($host =~ /^(.*)@([^@]+)$/);
   ($host, $port, $timeout, $retries, $backoff, $version) = split(':', $host, 6)
     if ($host =~ /:/);
   $version = '1' unless defined $version;
-  $port = 161 if (defined($port) && (length($port) <= 0));
-  $nhost = "$community\@$host:$port";
+  if ($port =~ /^([^!]*)!(.*)$/)
+  {
+    ($port, $lhost) = ($1, $2);
+    $nlhost = $lhost;
+    ($lhost, $lport) = ($1, $2) if ($lhost =~ /^(.*)!(.*)$/);
+    undef($lhost) if (defined($lhost) && (length($lhost) <= 0));
+    undef($lport) if (defined($lport) && (length($lport) <= 0));
+  }
+  undef($port) if (defined($port) && length($port) <= 0);
+  $port = 162 if ($type == 1 && !defined($port));
+  $nhost = "$community\@$host";
+  $nhost .= ":" . $port if (defined($port));
 
   if ((!defined($SNMP_util::Session))
     || ($SNMP_util::Host ne $nhost)
-      || ($SNMP_util::Version ne $version))
+    || ($SNMP_util::Version ne $version)
+    || ($SNMP_util::LHost ne $nlhost))
   {
     if (defined($SNMP_util::Session))
     {
@@ -368,17 +385,50 @@ sub snmpopen (@) {
       undef $SNMP_util::Session;
       undef $SNMP_util::Host;
       undef $SNMP_util::Version;
+      undef $SNMP_util::LHost;
     }
-    $SNMP_util::Session =
-	($version =~ /^2c?/)
-	    ? SNMPv2c_Session->open($host, $community, $port)
-		: SNMP_Session->open($host, $community, $port);
-    ($SNMP_util::Host = $nhost, $SNMP_util::Version = $version)
-	if defined($SNMP_util::Session);
+    $SNMP_util::Session = ($version =~ /^2c?$/i)
+      ? SNMPv2c_Session->open($host, $community, $port, undef,
+				$lport, undef, $lhost)
+      : SNMP_Session->open($host, $community, $port, undef,
+				$lport, undef, $lhost);
+    ($SNMP_util::Host = $nhost, $SNMP_util::Version = $version,
+      $SNMP_util::LHost = $nlhost) if defined($SNMP_util::Session);
   }
 
   if (defined($SNMP_util::Session))
   {
+    if (ref $vars->[0] eq 'HASH')
+    {
+	my $opts = shift @$vars;
+	foreach $type (keys %$opts)
+	{
+	    if (exists $SNMP_util::Session->{$type})
+	    {
+		if ($type eq 'timeout')
+		{
+		    $SNMP_util::Session->set_timeout($opts->{$type});
+		}
+		elsif ($type eq 'retries')
+		{
+		    $SNMP_util::Session->set_retries($opts->{$type});
+		}
+		elsif ($type eq 'backoff')
+		{
+		    $SNMP_util::Session->set_backoff($opts->{$type});
+		}
+		else
+		{
+		    $SNMP_util::Session->{$type} = $opts->{$type};
+		}
+	    }
+	    else
+	    {
+		carp "SNMPopen Unknown SNMP Option Key '$type'\n"
+		    unless ($SNMP_Session::suppress_warnings > 1);
+	    }
+	}
+    }
     $SNMP_util::Session->set_timeout($timeout)
       if (defined($timeout) && (length($timeout) > 0));
     $SNMP_util::Session->set_retries($retries)
@@ -398,9 +448,9 @@ sub snmpget (@) {
   my(@enoid, $var, $response, $bindings, $binding, $value, $oid, @retvals);
   my $session;
 
-  $session = &snmpopen($host);
+  $session = &snmpopen($host, 0, \@vars);
   if (!defined($session)) {
-    warn "SNMPGET Problem for $host\n"
+    carp "SNMPGET Problem for $host\n"
       unless ($SNMP_Session::suppress_warnings > 1);
     return undef;
   }
@@ -420,7 +470,7 @@ sub snmpget (@) {
     return(@retvals);
   }
   $var = join(' ', @vars);
-  warn "SNMPGET Problem for $var on $host\n"
+  carp "SNMPGET Problem for $var on $host\n"
     unless ($SNMP_Session::suppress_warnings > 1);
   return undef;
 }
@@ -435,9 +485,9 @@ sub snmpgetnext (@) {
   my($noid, $ok);
   my $session;
 
-  $session = &snmpopen($host);
+  $session = &snmpopen($host, 0, \@vars);
   if (!defined($session)) {
-    warn "SNMPGETNEXT Problem for $host\n"
+    carp "SNMPGETNEXT Problem for $host\n"
       unless ($SNMP_Session::suppress_warnings > 1);
     return undef;
   }
@@ -467,7 +517,7 @@ sub snmpgetnext (@) {
   else
   {
     $var = join(' ', @vars);
-    warn "SNMPGETNEXT Problem for $var on $host\n"
+    carp "SNMPGETNEXT Problem for $var on $host\n"
       unless ($SNMP_Session::suppress_warnings > 1);
     return undef;
   }
@@ -483,9 +533,9 @@ sub snmpwalk (@) {
   my($got, @nnoid, $noid, $ok);
   my $session;
 
-  $session = &snmpopen($host);
+  $session = &snmpopen($host, 0, \@vars);
   if (!defined($session)) {
-    warn "SNMPWALK Problem for $host\n"
+    carp "SNMPWALK Problem for $host\n"
       unless ($SNMP_Session::suppress_warnings > 1);
     return undef;
   }
@@ -539,7 +589,7 @@ sub snmpwalk (@) {
   else
   {
     $var = join(' ', @vars);
-    warn "SNMPWALK Problem for $var on $host\n"
+    carp "SNMPWALK Problem for $var on $host\n"
       unless ($SNMP_Session::suppress_warnings > 1);
     return undef;
   }
@@ -554,10 +604,10 @@ sub snmpset(@) {
     my($oid, @retvals, $type, $value);
     my $session;
 
-    $session = &snmpopen($host);
+    $session = &snmpopen($host, 0, \@vars);
     if (!defined($session))
     {
-	warn "SNMPSET Problem for $host\n"
+	carp "SNMPSET Problem for $host\n"
 	    unless ($SNMP_Session::suppress_warnings > 1);
 	return undef;
     }
@@ -590,7 +640,7 @@ sub snmpset(@) {
 	}
 	else
 	{
-	    warn "unknown SNMP type: $type\n"
+	    carp "unknown SNMP type: $type\n"
 		unless ($SNMP_Session::suppress_warnings > 1);
 	    return undef;
 	}
@@ -620,11 +670,10 @@ sub snmptrap(@) {
     my(@enoid);
     my $session;
 
-    $host = $host . ':162' if !($host =~ /:/);
-    $session = &snmpopen($host);
+    $session = &snmpopen($host, 1, \@vars);
     if (!defined($session))
     {
-	warn "SNMPTRAP Problem for $host\n"
+	carp "SNMPTRAP Problem for $host\n"
 	    unless ($SNMP_Session::suppress_warnings > 1);
 	return undef;
     }
@@ -665,7 +714,7 @@ sub snmptrap(@) {
 	}
 	else
 	{
-	    warn "unknown SNMP type: $type\n"
+	    carp "unknown SNMP type: $type\n"
 		unless ($SNMP_Session::suppress_warnings > 1);
 	    return undef;
 	}
@@ -719,7 +768,7 @@ sub toOID(@)
 	if ($oid) {
 	    $var =~ s/^$tmp/$oid/;
 	} else {
-	    warn "Unknown SNMP var $var\n"
+	    carp "Unknown SNMP var $var\n"
 		unless ($SNMP_Session::suppress_warnings > 1);
 	    next;
 	}
@@ -769,7 +818,7 @@ sub snmpLoad_OID_Cache ($)
 
     if (!open(CACHE, $arg))
     {
-	warn "snmpLoad_OID_Cache: Can't open $arg: $!"
+	carp "snmpLoad_OID_Cache: Can't open $arg: $!"
 	    unless ($SNMP_Session::suppress_warnings > 1);
 	return -1;
     }
@@ -858,7 +907,7 @@ sub snmpMIB_to_OID ($)
 
     if (!open(MIB, $arg))
     {
-	warn "snmpMIB_to_OID: Can't open $arg: $!"
+	carp "snmpMIB_to_OID: Can't open $arg: $!"
 	    unless ($SNMP_Session::suppress_warnings > 1);
 	return -1;
     }
@@ -919,7 +968,7 @@ sub snmpMIB_to_OID ($)
 	    $buf =~ s/ +$//;
 	    ($code, $val) = split(' ', $buf, 2);
 
-	    if (length($val) <= 0)
+	    if (!defined($val) || (length($val) <= 0))
 	    {
 		$SNMP_util::OIDS{$var} = $code;
 		$ret++;
@@ -957,7 +1006,7 @@ sub snmpMIB_to_OID ($)
 
 		if (!defined($SNMP_util::OIDS{$strt}))
 		{
-		    warn "snmpMIB_to_OID: $arg: \"$strt\" prefix unknown, load the parent MIB first.\n"
+		    carp "snmpMIB_to_OID: $arg: \"$strt\" prefix unknown, load the parent MIB first.\n"
 			unless ($SNMP_Session::suppress_warnings > 1);
 		}
 		$Link{$var} = $strt;
@@ -986,7 +1035,7 @@ sub encode_oid_with_errmsg ($) {
     my ($oid) = @_;
     my $tmp = encode_oid(split(/\./, $oid));
     if (! defined $tmp) {
-	warn "cannot encode Object ID $oid: $BER::errmsg"
+	carp "cannot encode Object ID $oid: $BER::errmsg"
 	    unless ($SNMP_Session::suppress_warnings > 1);
 	return undef;
     }
