@@ -1,5 +1,5 @@
 /*
- MRTG 2.9.10  -- Rateup
+ MRTG 2.9.12a  -- Rateup
  *********************
 
  Rateup is a fast add-on to the great MRTG Traffic monitor.  It makes
@@ -27,6 +27,7 @@
             which was fixed when recompiled with VC++ 6.0
 			Alexandre Steinberg
 			steinberg@base.com.br
+
 */
 
 #include <stdlib.h>
@@ -60,13 +61,20 @@
 #include <gd.h>
 #include <gdfonts.h>
 
-char *VERSION = "2.9.10";
+char *VERSION = "2.9.12a";
 char *program,*router,*routerpath;
 int histvalid;
-int dorelpercent = 0;
-int transparent = 0;
-int unknaszero = 0;
-int withzeroes = 0;
+
+/* Options */
+short options = 0;
+#define OPTION_WITHZEROES	0x0001	/* withzeros */
+#define OPTION_UNKNASZERO	0x0002	/* unknaszero */
+#define OPTION_TRANSPARENT	0x0004	/* transparent */
+#define OPTION_DORELPERCENT	0x0008	/* dorelpercent */
+#define OPTION_NOBORDER		0x0010	/* noborder */
+#define OPTION_NOARROW		0x0020	/* noarrow */
+#define OPTION_NO_I		0x0040	/* ignore 'I' (first) variable */
+#define OPTION_NO_O		0x0080	/* ignore 'O' (second) variable */
 
 time_t NOW;
 
@@ -231,7 +239,7 @@ int peak;
     struct HISTORY *lhist;
     /* ################################################# */
     /* Some general definitions for the graph generation */
-#define XSIZE (unsigned long)((maxx*xscale)+100+dorelpercent*30)
+#define XSIZE (unsigned long)((maxx*xscale)+100+((options & OPTION_DORELPERCENT) ? 1 : 0)*30)
 #define YSIZE (unsigned long)((maxy*yscale)+35)
     /* position the graph */
 #define ytr(y) (unsigned long)(maxy*yscale+14-((y)*yscale))
@@ -282,7 +290,7 @@ int peak;
 	/*scale with muli */
 	inr *=muli; outr *=muli; inmax *=muli; outmax *=muli;	
 	/* ignore times when we have not sampled */
-	if(inr>0 || outr>0 || withzeroes == 1) valsamp++;
+	if(inr>0 || outr>0 || (options & OPTION_WITHZEROES)) valsamp++;
 	if (x==0) {latestin=inr;latestout=outr;if (outr) {latestpercent=inr*(double)100./outr;}}
 	avin += inr;
 	avout += outr;
@@ -297,13 +305,13 @@ int peak;
 		maxs = max(maxs,inr);
 		maxs = max(maxs,outr);	
 	}
-	if (dorelpercent && outr) {
+	if ((options & OPTION_DORELPERCENT) && outr) {
 		dummy = (double)100.*inr/outr;
 		maxpercent = max(dummy,maxpercent);
 	}
 	now -= step;
     }
-    if (dorelpercent) {
+    if (options & OPTION_DORELPERCENT) {
        if (avout && valsamp) {
           avpercent = (double)100.*avin/avout;
        }
@@ -318,17 +326,17 @@ int peak;
 	
     printf("%lu\n",(unsigned long)(maxin/muli));
     printf("%lu\n",(unsigned long)(maxout/muli));
-    if (dorelpercent) {
+    if (options & OPTION_DORELPERCENT) {
        printf("%lu\n",(unsigned long)(maxpercent));
     }
     printf("%lu\n",(unsigned long)(avin/muli));
     printf("%lu\n",(unsigned long)(avout/muli));
-    if (dorelpercent) {
+    if (options & OPTION_DORELPERCENT) {
        printf("%lu\n",(unsigned long)(avpercent));
     }
     printf("%lu\n",(unsigned long)(latestin/muli));
     printf("%lu\n",(unsigned long)(latestout/muli));
-    if (dorelpercent) {
+    if (options & OPTION_DORELPERCENT) {
        printf("%lu\n",(unsigned long)(latestpercent));
     }
 
@@ -359,6 +367,8 @@ int peak;
     digits = 0;
     {
     double number = (double)maxv_q;
+    number *= yticsf;
+
 /*
        while (number/(double) kilo >= (double)kilo && digits<kMGnumber*3) {
 */
@@ -520,7 +530,7 @@ int peak;
 	}
         lhist[x].outmax = y;
 
-	if (dorelpercent ) {
+	if (options & OPTION_DORELPERCENT) {
 	   if (outr != (unsigned long)0) {
               percent = (double)inr/(double)outr;
 /*
@@ -555,7 +565,7 @@ int peak;
     i_light = gdImageColorAllocate(graph,c_light);
     i_dark = gdImageColorAllocate(graph,c_dark);
 
-    if (transparent) {
+    if (options & OPTION_TRANSPARENT) {
        gdImageColorTransparent(graph, i_blank);
     }
     gdImageInterlace(graph, 1); 
@@ -570,6 +580,7 @@ int peak;
     i_outpg = gdImageColorAllocate(graph,col_outp[0], col_outp[1], col_outp[2]);
     
     /* draw the image border */
+   if (!(options & OPTION_NOBORDER)) {
     gdImageLine(graph,0,0,XSIZE-1,0,i_light);
     gdImageLine(graph,1,1,XSIZE-2,1,i_light);
     gdImageLine(graph,0,0,0,YSIZE-1,i_light);
@@ -578,9 +589,10 @@ int peak;
     gdImageLine(graph,0,YSIZE-1,XSIZE-1,YSIZE-1,i_dark);
     gdImageLine(graph,XSIZE-2,1,XSIZE-2,YSIZE-2,i_dark);
     gdImageLine(graph,1,YSIZE-2,XSIZE-2,YSIZE-2,i_dark);
-
+   }
 
     /* draw the incoming traffic */
+   if (!(options & OPTION_NO_I)) {
     for (x=0; x<maxx; x++) {
       /* peak is always above average, we therefore only draw the upper part */
       if (peak) gdImageLine(graph,xtr(x),ytr(lhist[x].in),
@@ -594,9 +606,10 @@ int peak;
       gdImageLine(graph,xtr(x+0.5),ytr(0),xtr(x+0.5),ytr(lhist[x].in),i_in);
 #endif
     }
+   }
 
     /* draw the percentage */
-    if (dorelpercent) {
+    if (options & OPTION_DORELPERCENT) {
       gdImageSetBrush(graph, brush_outp);
       for (x=0; x<maxx-1; x++) 
          gdImageLine(graph,xtr(x), ytr(lhist[x].percent),
@@ -605,6 +618,7 @@ int peak;
     }
  
     /* draw the outgoing traffic */
+   if (!(options & OPTION_NO_O)) {
     gdImageSetBrush(graph, brush_outm);
     if (peak) for (x=0; x<maxx-1; x++) 
        gdImageLine(graph,xtr(x), ytr(lhist[x].outmax),
@@ -615,11 +629,10 @@ int peak;
       gdImageLine(graph,xtr(x),ytr(lhist[x].out),
 		  xtr(x+1),ytr(lhist[x+1].out),
 		  gdBrushed);
+   }
  
     /* draw the graph border */
-    
-    gdImageRectangle(graph,
-		     xtr(0),ytr(0),xtr(maxx),ytr(maxy),i_grid);
+    gdImageRectangle(graph,xtr(0),ytr(0),xtr(maxx),ytr(maxy),i_grid);
     
     /*create a dotted style for the grid lines*/
     styleDotted[0] = i_grid;
@@ -652,13 +665,14 @@ int peak;
 	
 /*
     	sprintf(ylab,"%6.1f %s",sca_max_q*i,short_si[digits/3]);
-*/
     	sprintf(ylab,"%6.1f %s",sca_max_q*i*yticsf,short_si_out);
+*/
+        sprintf(ylab,"%6.1f %s",sca_max_q*i,short_si_out);
         gdImageString(graph, gdFontSmall,23, 
         	      ytr(i*maxy/ytics+gdFontSmall->h/2),
 		      (unsigned char*)ylab, i_grid);
 
-	if (dorelpercent) {	
+	if (options & OPTION_DORELPERCENT) {	
            /* sprintf(ylab,"% 6.1f%%",(float)25.*i); */
            sprintf(ylab,"% 6.1f%%",(float)100/ytics*i);
  	   gdImageString(graph, gdFontSmall,77+((maxx)*xscale)+1,
@@ -706,12 +720,14 @@ int peak;
     }
 
     /* draw a red arrow a 0,0 */
+   if (!(options & OPTION_NOARROW)) {
     gdImageLine(graph, xtr(2),ytr(3),xtr(2),ytr(-3), i_major);
     gdImageLine(graph, xtr(1),ytr(3),xtr(1),ytr(-3), i_major);
     gdImageLine(graph, xtr(0),ytr(2),xtr(0),ytr(-2), i_major);
     gdImageLine(graph, xtr(-1),ytr(1),xtr(-1),ytr(-1), i_major);
     gdImageLine(graph, xtr(-2),ytr(1),xtr(-2),ytr(-1), i_major);
     gdImageLine(graph, xtr(-3),ytr(0),xtr(-3),ytr(0), i_major);
+   }
 
 
     if ((fo = fopen(file,"wb")) == NULL) {
@@ -734,7 +750,12 @@ char *a,*b;
 {
     char res[MAXL], *a1, *b1, *r1;
     int c,x,m;
-
+    if (*a == '-' && *b == '-') {
+       b1 = b + 1;
+       b = a + 1;
+       a = b1;
+    }
+    
     while (!isdigit(*a))
 	a++;
     while (!isdigit(*b))
@@ -928,7 +949,7 @@ int absupdate;
 
     period = ( now - last.time);
     if (period <= 0 || period > (60*60) || last.in[0] == 'x') {	/* if last update is strange */
-      if (unknaszero == 1) {
+      if (options & OPTION_UNKNASZERO) {
 	inrate = 0;                             /* sync unknown to zero */
 	outrate = 0;
       } else {
@@ -957,14 +978,14 @@ int absupdate;
 	}
     }
     if (inrate < 0 || inrate > abs_max) {
-      if (unknaszero == 1) {
+      if (options & OPTION_UNKNASZERO) {
 	inrate = 0;                             /* sync unknown to zero */
       } else {
 	inrate = history[0].in;			/* Sync by using last value */
       }
     }
     if (outrate < 0 || outrate > abs_max) {
-      if (unknaszero == 1) {
+      if (options & OPTION_UNKNASZERO) {
 	outrate = 0;                             /* sync unknown to zero */
       } else {
 	outrate = history[0].out;			/* Sync by using last value */
@@ -1149,7 +1170,7 @@ char **argv;
     if (argc < 2) {
 	fprintf(stderr,"%s for mrtg %s\n"
 		"Usage: %s directory basename [sampletime] [t sampletime] "
-               "[-(t)ransparent] "
+		"[-(t)ransparent] [-(b)order]"
 		"[u|a|g|h|m in out abs_max] "
 		"[i/p file maxvi maxvo maxx maxy growright step bits]\n",
 		program,VERSION,program);
@@ -1202,25 +1223,57 @@ char **argv;
 	switch(argv[argi][0]) {
            case '-':   /* -options */
                switch(argv[argi][1]) {
+	 	   case 'a':   /* Turn off the direction arrow */
+		       options |= OPTION_NOARROW;
+		       used = 1;
+		       break;
+		   case 'A':   /* Turn on the direction arrow */
+		       options &= ~OPTION_NOARROW;
+		       used = 1;
+		       break;
+	 	   case 'b':   /* Turn off the shaded border */
+		       options |= OPTION_NOBORDER;
+		       used = 1;
+		       break;
+		   case 'B':   /* Turn on the shaded border */
+		       options &= ~OPTION_NOBORDER;
+		       used = 1;
+		       break;
+	 	   case 'i':   /* Do not graph the I variable */
+		       options |= OPTION_NO_I;
+		       used = 1;
+		       break;
+		   case 'I':   /* Graph the I variable */
+		       options &= ~OPTION_NO_I;
+		       used = 1;
+		       break;
+	 	   case 'o':   /* Do not graph the O variable */
+		       options |= OPTION_NO_O;
+		       used = 1;
+		       break;
+		   case 'O':   /* Graph the O variable */
+		       options &= ~OPTION_NO_O;
+		       used = 1;
+		       break;
                    case 't':   /* Transparent Image */
-                       transparent = 1;
+                       options |= OPTION_TRANSPARENT;
                        used = 1;
                        break;
                    case 'T':   /* non-Transparent Image */
-                       transparent = 0;
+                       options &= ~OPTION_TRANSPARENT;
                        used = 1;
                        break;
 	           case 'z':   /* unknown as zero */
+	               options |= OPTION_UNKNASZERO;
 	               used = 1;
-		       unknaszero = 1;
 		       break;
 	           case 'Z':   /* repeat last */
+	               options &= ~OPTION_UNKNASZERO;
 	               used = 1;
-		       unknaszero = 0;
 		       break;
 	           case '0':  /* assume zeroes */
+	               options |= OPTION_UNKNASZERO;
 	               used = 1;
-	               withzeroes = 1;
 	               break;	
                    default:
                        fprintf(stderr,"Rateup ERROR: Unknown option: %s, sorry!\n",argv[argi]);
@@ -1266,35 +1319,35 @@ char **argv;
 		}
 	    case 'u':	/* Update file */
                 if (argv[argi][1] == 'p') {
-		   dorelpercent = 1;
+		   options |= OPTION_DORELPERCENT;
                 }
 		update(argv[argi+1],argv[argi+2],atol(argv[argi+3]),0);
 		used = 4;
 		break;
 	    case 'a':	/* Absolute Update file */
                 if (argv[argi][1] == 'p') {
-		   dorelpercent = 1;
+		   options |= OPTION_DORELPERCENT;
                 }
 	   	update(argv[argi+1],argv[argi+2],atol(argv[argi+3]),1);
 		used = 4;
 		break;
 	    case 'g':	/* Gauge Update file */
                 if (argv[argi][1] == 'p') {
-		   dorelpercent = 1;
+		   options |= OPTION_DORELPERCENT;
                 }
 		update(argv[argi+1],argv[argi+2],atol(argv[argi+3]),2);
 		used = 4;
 		break;
             case 'h':
                 if (argv[argi][1] == 'p') {
-		   dorelpercent = 1;
+		   options |= OPTION_DORELPERCENT;
                 }
                 update(argv[argi+1],argv[argi+2],atol(argv[argi+3]),3);
                 used = 4;
                 break;
             case 'm':
                 if (argv[argi][1] == 'p') {
-		   dorelpercent = 1;
+		   options |= OPTION_DORELPERCENT;
                 }
                 update(argv[argi+1],argv[argi+2],atol(argv[argi+3]),4);
                 used = 4;
